#!/usr/bin/env python
#
# gturing - A Turing Machine's simulator
# Copyright (C) 2007 by German Poo-Caamano <gpoo@gnome.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from gturing import __version__
from gturing.reader import Reader
from gturing.tape import BLANK
from gturing.gui import main_gui
import optparse

def no_gui(options, args=[]):
    if not options.program:
        print 'In no-gui mode, you must specifiy the program to run.'
        return 1

    r = Reader(options.program)
    machine = r.read_states()

    # Everything else belongs to the tape (except argv[0] which the
    # name of the program invoked (this code).
    machine.set_tape(BLANK.join(args[1:]))

    if options.verbose:
        print machine.get_comments()
        print machine

    # Reset the tape's position and machine state to run the program
    # loaded
    machine.reset()

    # Run the program and show its state step by step
    while machine.run_state():
        if options.verbose:
            print machine.get_tape()

    print 'Result tape: %s' % machine.get_tape()
    return

def gui(options, args=[]):
    main_gui(options, args)

def main(args=[]):
    option_parser = optparse.OptionParser(
            usage='usage: %prog [options] "<tape>"')
    option_parser.add_option('-n', '--no-gui',
            action='store_true', dest='no_gui', default=False,
            help='Run the simulator without a grahpical user interface.')
    option_parser.add_option('-v', '--verbose',
            action='store_true', dest='verbose', default=False,
            help='In no-gui mode, show step by step output.')
    option_parser.add_option('-p', '--program', 
            dest='program', metavar="FILE",
            help='Name of the file with the Turing program to run.')

    (options, args) = option_parser.parse_args(args)

    if options.no_gui:
        no_gui(options, args)
    else:
        gui(options, args)

# vi:set ts=4 sw=4 expandtab:
