# Copyright (C) 2007 German Poo-Caaman~o <gpoo@gnome.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import re
try:
    from gturing.turing_machine import TuringMachine, State
    from gturing.tape import BLANK
except:
    from turing_machine import TuringMachine, State
    from tape import BLANK

class TuringParser:
    comment_regex = re.compile('^#(.*)')
    states_regex = re.compile('^(\d+)\s+([\d_]+)\s+([\d_]+)'
                              '\s+([rRlL])\s+(\d+)\s*(.*)')
    def __init__(self):
        self.machine = TuringMachine()
        self.comment = ''

    def add_line(self, s):
        m = self.comment_regex.search(s)
        if m:
            self.comment += m.group(1) + '\n'
            return

        m = self.states_regex.search(s)
        if m:
            (id, read, write, move, next, comment) = m.groups()
            if read == '_':
                read = BLANK
            if write == '_':
                write = BLANK

            state = State( (id, read, write, move, next, comment) )
            self.machine.add_state(state)
            return

class Reader():
    def __init__(self, filename):
        self.filename = filename

    def read_states(self):
        parser = TuringParser()

        for line in file(self.filename, "r"):
            parser.add_line(line)

        parser.machine.set_comments(parser.comment)
        return parser.machine

if __name__ == "__main__":
    import sys

    r = Reader(sys.argv[1])
    machine = r.read_states()
    machine.set_tape(sys.argv[2])
    print machine.get_comments()
    print machine
    machine.reset()
    while machine.run_state():
        pass

# vi:set ts=4 sw=4 expandtab:
