# Copyright (C) 2007 German Poo-Caaman~o <gpoo@gnome.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

(LEFT, RIGHT, BLANK) = ('l', 'r', ' ')

class Tape:
    def __init__(self, tape):
        self.position = 0
        self.set_tape(tape)

    def set_tape(self, tape):
        self.tape = tape

    def get_tape(self):
        return self.tape

    def read(self):
        return self.tape[self.position]

    def write(self, value):
        pos = self.position
        self.tape = self.tape[:pos] + str(value) + self.tape[pos + 1:]

    def move(self, direction):
        direction = direction.lower()
        if direction == LEFT:
            if self.position >= 0:
                self.position -= 1
            else:
                self.position = 0
        else:
            self.position += 1

        # When we get the boundaries, we try to get out the range of
        # the tape, we add a new space giving the impression of
        # infinite
        if direction == LEFT and self.position < 0:
            self.tape = BLANK + self.tape
            self.position = 0

        if (direction == RIGHT and self.position == len(self.tape)):
            self.tape += BLANK

    def reset(self):
        self.position = 0

if __name__ == "__main__":
    import sys

    TAPE = '1 1001'

    tape = Tape(TAPE)
    print 'Tape: "%s"' % TAPE
    print "Moving the tape 3 positions left"
    for i in range(0, 3):
        tape.move(LEFT)
        str_tape = tape.get_tape()
        print ' "%s"' % str_tape

    print "Moving the tape %d positions right" % (len(str_tape) + 3)
    for i in range(0, len(str_tape)+3):
        tape.move(RIGHT)
        str_tape = tape.get_tape()
        print ' "%s"' % str_tape

    print 'Write 0 and move left 13 times'
    for i in range(0, 13):
        str_old_tape = tape.get_tape()
        value = tape.read()
        tape.write(0)
        str_new_tape = tape.get_tape()
        tape.move(LEFT)
        print ' "%15s"  %s==>0  "%15s"' % (str_old_tape, value, str_new_tape)
    str_tape = tape.get_tape()
    print '  %15s   %s==>   "%15s"' % ('Final position', ' ', str_tape)

    print 'Write 1 and move right 13 times'
    for i in range(0, 13):
        str_old_tape = tape.get_tape()
        value = tape.read()
        tape.write(1)
        str_new_tape = tape.get_tape()
        tape.move(RIGHT)
        print ' "%15s"  %s==>0  "%15s"' % (str_old_tape, value, str_new_tape)
    str_tape = tape.get_tape()
    print '  %15s   %s==>   "%15s"' % ('Final position', ' ', str_tape)

# vi:set ts=4 sw=4 expandtab:
