# Copyright (C) 2007 German Poo-Caaman~o <gpoo@gnome.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

try:
    from gturing import tape
except:
    import tape

class State:
    def __init__(self, states):
        self.id    = states[0]
        self.read  = states[1]
        self.write = states[2]
        self.move  = states[3]
        self.next  = states[4]
        self.comment = states[5]

    def __str__(self):
        """return "[%s]" % ', '.join(
            ['%s=%r' % (attr, getattr(self, attr)) for attr in 
             ['id', 'read', 'write', 'move', 'next_state', 'comment']])"""
        return ' '.join(
            ['%s' % getattr(self, attr) for attr in 
             ['id', 'read', 'write', 'move', 'next', 'comment']])

class TuringMachine:
    graph = {}
    index = 0
    tape = None
    def add_state(self, state):
        self.index += 1
        self.graph[self.index] = state

        # The initial state is the first one in the machine
        if self.index == 1:
            self.state = self.graph[1].id

    def __str__(self):
        return '\n'.join(
               ['%3s => %s' % (k, v) for (k, v) in self.graph.iteritems()])

    def reset(self):
        if not self.is_empty():
            self.state = self.graph[1].id
        else:
            self.state = None

        if self.tape:
            self.tape.reset()

    def is_empty(self):
        return len(self.graph) == 0

    def set_tape(self, new_tape):
        self.tape = tape.Tape(new_tape)

    def get_tape(self):
        return self.tape.get_tape()

    def set_comments(self, comments):
        self.comments = comments

    def get_comments(self):
        return self.comments

    def get_tape_position(self):
        if self.tape:
            return self.tape.position
        else:
            return 0

    def search_state(self, id):
        for (idx, state) in self.graph.iteritems():
            if state.id == id and idx > self.idx:
                self.idx = idx
                return state
        return None

    def run_state(self):
        current_state = self.graph[1]
        self.idx = 1

        if self.is_empty():
            return False

        while ((current_state is not None) and 
               ((current_state.id != self.state) or
                (current_state.read != self.tape.read() ))):
            current_state = self.search_state(self.state)

        if current_state is not None:
            self.tape.write(current_state.write)
            self.tape.move(current_state.move)
            self.state = current_state.next;
            return True

        return False

    def get_states(self):
        '''Get the states (vertices) of the Turing Machine'''
        return self.graph.keys()

if __name__ == "__main__":
    import sys

    r = reader(sys.argv[1])
    machine = r.read_states()
    machine.set_tape(sys.argv[2])
    print machine
    machine.reset()
    while machine.run_state():
        print machine.get_tape()
        pass

# vi:set ts=4 sw=4 expandtab:
